#!/usr/bin/perl

use strict;
use warnings;
no warnings 'uninitialized', 'once';
use Library_raptorfast;
use LWP::UserAgent;
use Digest::SHA qw(sha512_hex);
use URI::Escape;
use JSON;

$|++;

our ($VERSION) = "1.0";
our ($PROGRAM) = 'PCCC Raptor FAST Password Hook';
our ($COPYRIGHT) = 'Copyright (C) 2014 Peregrine Computer Consultants Corporation. All rights reserved.';

my (%FORM, $FORM);
$FORM = \%FORM;

print "$PROGRAM v$VERSION\n$COPYRIGHT\n\n";

if (-e "config") {
  print "Loading Configuration... \n";
  require "config";
  if ($main::token eq '') {
    print "Error.  You must have a valid Raptor FAST Token defined in your config file!\nExiting.\n\n";
    exit;
  }
} else {
  print "Error Loading Configuration! You must copy config.dist to config and edit it for your installation.\n";
  exit;
}

#This script requires two arguments to run
if (scalar @ARGV != 2) {
  print "ERROR: Insufficient Arguments\n\n";
  print "Usage: $0 username password\n";
  print "   'password' should be URL encoded for escaping purposes\n\n";
  die("This script is meant to be called by passwordhk.dll.\n\n");
}

my $nt_user = shift(@ARGV);
my $nt_password = shift(@ARGV);

#URL decode password
$nt_password =~ uri_unescape($nt_password);

#'%user%' is used as a placeholder in the email query
$main::email_filter =~ s/\%user\%/$nt_user/g;

&main($FORM, bind=>$main::bind, password=>$main::password, base=>$main::base,
		     servers=>\@main::servers, filter=>$main::email_filter, nt_password=>$nt_password, 
			 nt_user=>$nt_user, token=>$main::token, verbose=>1);
exit;


sub main {
  my ($FORM, %params) = @_;
  my ($ldap, %FORM, $rv, $status, $msg, $key, $output, $i);

  #CONNECT/LOGIN TO THE LDAP DATABASE
  ($rv, $status, $ldap) = &Library_raptorfast::ldap_connect_and_bind(\%FORM, servers=>$params{'servers'}, bind=>$params{'bind'}, password=>$params{'password'});
  if ($rv < 1) {
    print "Fatal $status\nExiting...\n\n";
    exit;
  }

  #SEARCH THE DIRECTORY
  our @accounts;
  ($rv, $status) = &Library_raptorfast::ldap_search($FORM, ldap=>$ldap, filter=>$params{'filter'}, base=>$params{'base'});
  #print "DEBUG: $rv - $status\n";
  
  #BUILD JSON OUTPUT
  my $json = { token=>$params{'token'}, action=>'change_password', version=>"2.0",
			   new_password=>sha512_hex($params{'nt_password'}), username=>$params{'nt_user'} };


  my $account;
  if ($rv) {
    $account = $accounts[0]; #SHOULD BE THE ONLY LDAP QUERY RESULT
  }
  
  use Data::Dumper;
  print "DEBUG: " . Dumper(@accounts) . "\n";
  
  $json->{'name'} = $account->{'name'};
  $json->{'primary_email'} = $account->{'primary_email'};

  #UNBIND/LOGOUT AND DISCONNECT FROM THE SERVER
  ($rv, $status) = &Library_raptorfast::ldap_unbind_and_disconnect($FORM, ldap=>$ldap);
  #print "$status\n";
  
  #CHECK THAT USER HAS EMAILS ASSOCIATED BEFORE SUBMITTING
  if ($account->{'primary_email'} eq '') {
	print "Skipping submission - no email address is associated with the user '$params{'nt_user'}'.\n\n";
	return 127;
  }

  #REPORT THE EMAILS AND THE NEW PASSWORD HASH TO THE RAPTOR SERVER
  if ($params{'verbose'}) {
	
	print Dumper($json);
  }
  #print $json if ($params{'verbose'});
  &submit_request_to_raptor($FORM, json=>$json, verbose=>$params{'verbose'});

  #TEST FOR SCHEDULING PURPOSES

}

sub submit_request_to_raptor {
  my ($FORM, %params) = @_;
  my ($user_agent, $request, $result, $json);
  
  $json = $params{'json'};

  #REPORT THE RESULTS TO THE RAPTOR SERVER
  $user_agent = new LWP::UserAgent;
  $request = new HTTP::Request("POST", "https://www.pccc.com/raptor_FAST_submit.cgim");

  $request->content_type("application/x-www-form-urlencoded");

  #URL ENCODE THE INFORMATION
  my $json_string = to_json($json);
  
  $request->content("version=2.0&json_request=$json_string");
  $result = $user_agent->request($request);
  if ($result->is_error) { 
	my $response;
    print "Error submitting LDAP information to Raptor.\n";
	print "Server returned error " . $result->code() . ": " . $result->message() . "\n";
	eval {
		$response = JSON::from_json($result->decoded_content());
	};
	print "SERVER RESPONSE: $response\n" if ($params{'verbose'});
	if ($result ne '') {
	    print $result->decoded_content() . "\n";
	}
  } else {
    print "Raptor submission succesful.\n";
  }
}
